AUI.Buffs = {}

AUI_BUFF_TYPE_NORMAL = 1
AUI_BUFF_TYPE_DEBUFF = 2
AUI_BUFF_TYPE_HOT = 3
AUI_BUFF_TYPE_DOT = 4
AUI_BUFF_ALIGNMENT_LEFT = 100
AUI_BUFF_ALIGNMENT_RIGHT = 101

AUI_BUFF_TYPE_PLAYER = 1001
AUI_DEBUFF_TYPE_PLAYER = 1002
AUI_BUFF_TYPE_TARGET = 1003
AUI_DEBUFF_TYPE_TARGET = 1004

local LMP = LibStub("LibMediaProvider-1.0")
local BUFF_SCENE_FRAGMENT = nil

local isLoaded = false
local previewBuffList = {}
local buffDataList = {}

local mainBuffControls = nil

local hasTarget = false

local currentTemplate = nil

local function OnMouseDown(_eventCode, _button, _ctrl, _alt, _shift, _control)
	if _button == 1 then
		_control:SetMovable(true)
		_control:StartMoving()
	end
end

local function OnMouseUp(_eventCode, _button, _ctrl, _alt, _shift, _control, _positionData)
	_, _positionData.point, _, _positionData.relativePoint, _positionData.offsetX, _positionData.offsetY = _control:GetAnchor()
	
	_control:SetMovable(false)
end

local function GetBuffList()	
	return buffDataList
end

local function GetUnitTagBuffList(_unitTag)	
	return buffDataList[_unitTag]
end

local function GetBuffControls(_unitTag, _effectType)	
	if not buffDataList[_unitTag] then
		buffDataList[_unitTag] = {}
	end
	
	if not buffDataList[_unitTag][_effectType] then
		buffDataList[_unitTag][_effectType] = {}
	end	

	return buffDataList[_unitTag][_effectType]
end

local function GetBuffCount(_unitTag, _effectType)	
	local count = 1
	local controls = GetBuffControls(_unitTag, _effectType)	
	for id, _ in pairs(controls) do
		count = count + 1
	end

	return count
end

local function SortBuffs(_unitTag, _effectType)
	local buffControlList = GetBuffControls(_unitTag, _effectType)
	if buffControlList then
		table.sort(buffControlList, 
		function(controlLeft, controlRight)
			if _unitTag == AUI_PLAYER_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_BUFF then
				sorting = AUI.Settings.Buffs.player_buff_sorting
			elseif _unitTag == AUI_PLAYER_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_DEBUFF then	
				sorting = AUI.Settings.Buffs.player_debuff_sorting	
			elseif _unitTag == AUI_TARGET_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_BUFF then
				sorting = AUI.Settings.Buffs.target_buff_sorting
			elseif _unitTag == AUI_TARGET_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_DEBUFF then
				sorting = AUI.Settings.Buffs.target_debuff_sorting
			end	

			if sorting == AUI_SORTING_SMALL_TO_LARGE then
				if controlLeft.buffData.EndTime < 0 and controlRight.buffData.EndTime < 0 then		
					if controlLeft.buffData.StartTime < controlRight.buffData.StartTime then
						return true
					end		
				elseif controlLeft.buffData.EndTime < 0 and controlRight.buffData.EndTime > 0 then
					return false
				elseif controlLeft.buffData.EndTime > 0 and controlRight.buffData.EndTime < 0 or controlLeft.buffData.EndTime < controlRight.buffData.EndTime then
					return true
				end
			elseif sorting == AUI_SORTING_LARGE_TO_SMALL then
				if controlLeft.buffData.EndTime < 0 and controlRight.buffData.EndTime < 0 then		
					if controlLeft.buffData.StartTime > controlRight.buffData.StartTime then
						return true
					end		
				elseif controlLeft.buffData.EndTime > 0 and controlRight.buffData.EndTime < 0 then
					return false
				elseif controlLeft.buffData.EndTime < 0 and controlRight.buffData.EndTime > 0 or controlLeft.buffData.EndTime > controlRight.buffData.EndTime then
					return true
				end
			end
	
			return false
		end)
	end
end

local function GetParentWindow(_unitTag, _effectType)
	local parent = nil

	if _unitTag == AUI_PLAYER_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_BUFF then
		parent = AUI_Buff_Player_Buffs
	elseif _unitTag == AUI_PLAYER_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_DEBUFF then	
		parent = AUI_Buff_Player_Debuffs	
	elseif _unitTag == AUI_TARGET_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_BUFF then
		parent = AUI_Buff_Target_Buffs
	elseif _unitTag == AUI_TARGET_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_DEBUFF then
		parent = AUI_Buff_Target_Debuffs
	end	
	
	return parent
end

local function SetPreviewLayout(_unitTag, _effectType)
	local alignment = nil
	
	local main = nil
	if _unitTag == AUI_PLAYER_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_BUFF then
		alignment = AUI.Settings.Buffs.player_buff_alignment
		main = AUI_Buff_Player_Buffs
	elseif _unitTag == AUI_PLAYER_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_DEBUFF then	
		alignment = AUI.Settings.Buffs.player_debuff_alignment
		main = AUI_Buff_Player_Debuffs
	elseif _unitTag == AUI_TARGET_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_BUFF then
		alignment = AUI.Settings.Buffs.target_buff_alignment
		main = AUI_Buff_Target_Buffs
	elseif _unitTag == AUI_TARGET_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_DEBUFF then
		alignment = AUI.Settings.Buffs.target_debuff_alignment
		main = AUI_Buff_Target_Debuffs
	end	

	local info = GetControl(main, "_Info")
	local arrow = GetControl(main, "_DirectionArrow")

	arrow:ClearAnchors()
	info:ClearAnchors()

	if alignment == AUI_VERTICAL then
		arrow:SetTextureRotation(math.rad(180))		
		arrow:SetAnchor(BOTTOM, main, TOP, 0, -10)	
		info:SetAnchor(BOTTOM, arrow, TOP, 0, -4)			
	elseif alignment == AUI_VERTICAL_REVERSE then
		arrow:SetTextureRotation(math.rad(0))		
		arrow:SetAnchor(TOP, main, BOTTOM, 0, 24)		
		info:SetAnchor(TOP, arrow, BOTTOM, 0, 4)			
	elseif alignment == AUI_HORIZONTAL then
		arrow:SetTextureRotation(math.rad(270))		
		arrow:SetAnchor(RIGHT, main, LEFT, -10)	
		info:SetAnchor(RIGHT, arrow, LEFT, -4, -2)	
	elseif alignment == AUI_HORIZONTAL_REVERSE then		
		arrow:SetTextureRotation(math.rad(90))		
		arrow:SetAnchor(LEFT, main, RIGHT, 10)	
		info:SetAnchor(LEFT, arrow, RIGHT, 4, -2)			
	end		
end

local function SetBuffsLayout(_unitTag, _effectType)
	local alignment = nil
	local size = 32
	local color = "#ffffff"
	local distance = 12
	local fontArt = "Univers 55"
	local fontSize = 11
	local showBuffTime = true
	
	if _unitTag == AUI_PLAYER_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_BUFF then
		size = AUI.Settings.Buffs.player_buff_size
		alignment = AUI.Settings.Buffs.player_buff_alignment
		distance = AUI.Settings.Buffs.player_buff_distance
		fontArt = AUI.Settings.Buffs.player_buff_font_art
		fontSize =  AUI.Settings.Buffs.player_buff_font_size
		showBuffTime = AUI.Settings.Buffs.player_show_buff_time 
	elseif _unitTag == AUI_PLAYER_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_DEBUFF then	
		size = AUI.Settings.Buffs.player_debuff_size
		alignment = AUI.Settings.Buffs.player_debuff_alignment
		fontArt = AUI.Settings.Buffs.player_debuff_font_art
		fontSize =  AUI.Settings.Buffs.player_debuff_font_size
		showBuffTime = AUI.Settings.Buffs.player_show_debuff_time 		
		distance = AUI.Settings.Buffs.player_debuff_distance
	elseif _unitTag == AUI_TARGET_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_BUFF then
		size = AUI.Settings.Buffs.target_buff_size
		alignment = AUI.Settings.Buffs.target_buff_alignment
		distance = AUI.Settings.Buffs.target_buff_distance
		fontArt = AUI.Settings.Buffs.target_buff_font_art
		fontSize =  AUI.Settings.Buffs.target_buff_font_size
		showBuffTime = AUI.Settings.Buffs.target_show_buff_time 		
	elseif _unitTag == AUI_TARGET_UNIT_TAG and _effectType == BUFF_EFFECT_TYPE_DEBUFF then
		size = AUI.Settings.Buffs.target_debuff_size
		alignment = AUI.Settings.Buffs.target_debuff_alignment
		distance = AUI.Settings.Buffs.target_debuff_distance
		fontArt = AUI.Settings.Buffs.target_debuff_font_art
		fontSize =  AUI.Settings.Buffs.target_debuff_font_size
		showBuffTime = AUI.Settings.Buffs.target_show_debuff_time		
	end	


	local lastControl = nil
	local buffControls = GetBuffControls(_unitTag, _effectType, true)

	local width = 0
	local height = 0
	for _, control in pairs(buffControls) do
		if control.buffData.active then	
			local distanceLeft = 0
			local distanceTop = 0
		
			local fontSize = (size + fontSize) / 4.5
	
			local iconTexture = control.buffData.IconTexture	
			local iconTextureControl = GetControl(control, "_Icon")
			iconTextureControl:SetTexture(iconTexture)
			iconTextureControl:SetDimensions(size, size)			

			local textControl = GetControl(control, "_Text")
			if textControl then
				textControl:SetFont(LMP:Fetch('font', fontArt) .. "|" .. fontSize .. "|" .. "soft-shadow-thick")
				
				if showBuffTime then
					textControl:SetHidden(false)
				else
					textControl:SetHidden(true)
				end				
			end
	
			control:SetDimensions(size, size)	
			control:ClearAnchors()
	
			local point = TOPLEFT
			local rPoint = TOPRIGHT
			
			if alignment == AUI_VERTICAL then
				point = TOPLEFT
				rPoint = BOTTOMLEFT
				distanceTop = distance	
			elseif alignment == AUI_VERTICAL_REVERSE then
				point = BOTTOMLEFT
				rPoint = TOPLEFT
				distanceTop = -distance		
			elseif alignment == AUI_HORIZONTAL then
				point = TOPLEFT
				rPoint = TOPRIGHT
				distanceLeft = distance	
			elseif alignment == AUI_HORIZONTAL_REVERSE then
				point = TOPRIGHT
				rPoint = TOPLEFT
				distanceLeft = -distance				
			end
			
			if lastControl then
				control:SetAnchor(point, lastControl, rPoint, distanceLeft, distanceTop)			
			else
				control:SetAnchor(point)	
			end			

			lastControl = control
		else
			control:SetHidden(true)
		end	
	end	
	
	local parent = GetParentWindow(_unitTag, _effectType)
	parent:SetDimensions(size, size)	
end

local function CreateNewBuffControl(_virtualControl, _buffName, _unitTag, _effectType)
	local controls = GetBuffControls(_unitTag, _effectType)	
	local controlId = GetBuffCount(_unitTag, _effectType)
	local control = controls[controlId]

	if not control then
		local parent = GetParentWindow(_unitTag, _effectType)			
		if parent then
			control = CreateControlFromVirtual("AUI_Buff" .. _unitTag .. _effectType, parent, _virtualControl, controlId)
			control.buffData = {}
			control.buffData.IconTexture = ""
			control.buffData.StartTime =0
			control.buffData.EndTime = 0
			control.buffData.StartDuration = 0
			control.buffData.active = false									
			control.buffData.parent = parent
			control.buffData.UnitTag = _unitTag
			control.buffData.EffectType = _effectType
		
			controls[controlId] = control
		end
	end	

	return control
end

local function GetBuffControl(_buffName, _unitTag, _effectType)
	if not currentTemplate and currentTemplate.templateData then
		return nil
	end

	local controls = GetBuffControls(_unitTag, _effectType)	

	for id, control in pairs(controls) do			
		if control.buffData.BuffName == _buffName then	
			return control
		end
	end

	for id, control in pairs(controls) do
		if not control.buffData.active then
			control.buffData.IconTexture = ""
			control.buffData.StartTime = 0
			control.buffData.EndTime = 0
			control.buffData.BuffName = _buffName
			control.buffData.StartDuration = 0
			control.buffData.active = false	
			
			return control
		end
	end		

	if _unitTag == AUI_PLAYER_UNIT_TAG then
		if _effectType == BUFF_EFFECT_TYPE_BUFF then
			return CreateNewBuffControl(currentTemplate.templateData[AUI_BUFF_TYPE_PLAYER].name, _buffName, _unitTag, _effectType)
		elseif _effectType == BUFF_EFFECT_TYPE_DEBUFF then
			return CreateNewBuffControl(currentTemplate.templateData[AUI_DEBUFF_TYPE_PLAYER].name, _buffName, _unitTag, _effectType)
		end		
	elseif _unitTag == AUI_TARGET_UNIT_TAG then
		if _effectType == BUFF_EFFECT_TYPE_BUFF then
			return CreateNewBuffControl(currentTemplate.templateData[AUI_BUFF_TYPE_TARGET].name, _buffName, _unitTag, _effectType)
		elseif _effectType == BUFF_EFFECT_TYPE_DEBUFF then
			return CreateNewBuffControl(currentTemplate.templateData[AUI_DEBUFF_TYPE_TARGET].name, _buffName, _unitTag, _effectType)
		end		
	end
	
	return nil
end

local function CreateUI()
	AUI_Buff_Player_Buffs:ClearAnchors()
	AUI_Buff_Player_Buffs:SetAnchor(AUI.Settings.Buffs.player_buff_position.point, GUIROOT, AUI.Settings.Buffs.player_buff_position.relativePoint, AUI.Settings.Buffs.player_buff_position.offsetX, AUI.Settings.Buffs.player_buff_position.offsetY)		
	AUI_Buff_Player_Buffs:SetHandler("OnMouseDown", function(_eventCode, _button, _ctrl, _alt, _shift) OnMouseDown(_eventCode, _button, _ctrl, _alt, _shift, AUI_Buff_Player_Buffs) end)
	AUI_Buff_Player_Buffs:SetHandler("OnMouseUp", function(_eventCode, _button, _ctrl, _alt, _shift) OnMouseUp(_eventCode, _button, _ctrl, _alt, _shift, AUI_Buff_Player_Buffs, AUI.Settings.Buffs.player_buff_position) end)

	AUI_Buff_Player_Debuffs:ClearAnchors()
	AUI_Buff_Player_Debuffs:SetAnchor(AUI.Settings.Buffs.player_debuff_position.point, GUIROOT, AUI.Settings.Buffs.player_debuff_position.relativePoint, AUI.Settings.Buffs.player_debuff_position.offsetX, AUI.Settings.Buffs.player_debuff_position.offsetY)	
	AUI_Buff_Player_Debuffs:SetHandler("OnMouseDown", function(_eventCode, _button, _ctrl, _alt, _shift) OnMouseDown(_eventCode, _button, _ctrl, _alt, _shift, AUI_Buff_Player_Debuffs) end)
	AUI_Buff_Player_Debuffs:SetHandler("OnMouseUp", function(_eventCode, _button, _ctrl, _alt, _shift) OnMouseUp(_eventCode, _button, _ctrl, _alt, _shift, AUI_Buff_Player_Debuffs, AUI.Settings.Buffs.player_debuff_position) end)	
	
	AUI_Buff_Target_Buffs:ClearAnchors()
	AUI_Buff_Target_Buffs:SetAnchor(AUI.Settings.Buffs.target_buff_position.point, GUIROOT, AUI.Settings.Buffs.target_buff_position.relativePoint, AUI.Settings.Buffs.target_buff_position.offsetX, AUI.Settings.Buffs.target_buff_position.offsetY)	
	AUI_Buff_Target_Buffs:SetHandler("OnMouseDown", function(_eventCode, _button, _ctrl, _alt, _shift) OnMouseDown(_eventCode, _button, _ctrl, _alt, _shift, AUI_Buff_Target_Buffs) end)
	AUI_Buff_Target_Buffs:SetHandler("OnMouseUp", function(_eventCode, _button, _ctrl, _alt, _shift) OnMouseUp(_eventCode, _button, _ctrl, _alt, _shift, AUI_Buff_Target_Buffs, AUI.Settings.Buffs.target_buff_position) end)

	AUI_Buff_Target_Debuffs:ClearAnchors()
	AUI_Buff_Target_Debuffs:SetAnchor(AUI.Settings.Buffs.target_debuff_position.point, GUIROOT, AUI.Settings.Buffs.target_debuff_position.relativePoint, AUI.Settings.Buffs.target_debuff_position.offsetX, AUI.Settings.Buffs.target_debuff_position.offsetY)	
	AUI_Buff_Target_Debuffs:SetHandler("OnMouseDown", function(_eventCode, _button, _ctrl, _alt, _shift) OnMouseDown(_eventCode, _button, _ctrl, _alt, _shift, AUI_Buff_Target_Debuffs) end)
	AUI_Buff_Target_Debuffs:SetHandler("OnMouseUp", function(_eventCode, _button, _ctrl, _alt, _shift) OnMouseUp(_eventCode, _button, _ctrl, _alt, _shift, AUI_Buff_Target_Debuffs, AUI.Settings.Buffs.target_debuff_position) end)	
end

local function UpdateBuffTime(_buffName, _unitTag, _control)
	local timeLeft = _control.buffData.EndTime - GetFrameTimeSeconds()
	local duration = _control.buffData.EndTime - _control.buffData.StartTime

	local textControl = GetControl(_control, "_Text")
	if textControl then
		local formatedTimeString = ""
		if _control.buffData.StartDuration > 0 then
			if timeLeft >= 1 then
				formatedTimeString = AUI.Time.GetFormatedString(timeLeft, 0)
			else
				formatedTimeString = AUI.Time.GetFormatedString(timeLeft, 1)
			end
		else
			EVENT_MANAGER:UnregisterForUpdate("AUI_Update_BuffControl" .. tostring(_unitTag) .. _buffName)
		end	
		textControl:SetText(formatedTimeString)		
	end
		
	local cooldownControl = GetControl(_control, "_Cooldown")
	if cooldownControl then
		cooldownControl:StartCooldown(timeLeft * 1000, duration * 1000, CD_TYPE_RADIAL, CD_TIME_TYPE_TIME_UNTIL, false)
	end		
	
	if timeLeft <= 0 then
		EVENT_MANAGER:UnregisterForUpdate("AUI_Update_BuffControl" .. tostring(_unitTag) .. _buffName)	
	end
end

local function UpdateBuff(_buffName, _unitTag, _effectType, _iconTexture, _startTime, _endTime)
	EVENT_MANAGER:UnregisterForUpdate("AUI_Update_BuffControl" .. tostring(_unitTag) .. _buffName)	

	local control = GetBuffControl(_buffName, _unitTag, _effectType)
	local startDuration = _endTime - _startTime
	
	if startDuration <= 0 then
		_endTime = -1
		startDuration = -1
	end
	
	control.buffData.IconTexture = _iconTexture
	control.buffData.StartTime = _startTime or 0
	control.buffData.EndTime = _endTime or 0
	control.buffData.BuffName = _buffName
	control.buffData.StartDuration = startDuration
		
	EVENT_MANAGER:RegisterForUpdate("AUI_Update_BuffControl" .. tostring(_unitTag) .. _buffName, 0, function() UpdateBuffTime(_buffName, _unitTag, control) end)
end

local function IsBuffActive(_unitTag, _buffName, _effectType)
		local buffControls = GetBuffControls(_unitTag, _effectType)
	
		for _, control in pairs(buffControls) do
			local buffName = control.buffData.BuffName
			if buffName == _buffName and control.buffData.active then
				return true
			end
		end
	
	return false
end

local function AddBuff(_buffName, _unitTag, _effectType, _iconTexture, _startTime, _endTime, _fadeMode)
	
	queueBuffs =


	EVENT_MANAGER:UnregisterForUpdate("AUI_Update_BuffControl" .. tostring(_unitTag) .. _buffName)	

	local startDuration = _endTime - _startTime
		
	if startDuration <= 0 then
		_endTime = -1
		startDuration = -1
	end
		
	if not _iconTexture then
		_iconTexture = "/esoui/art/icons/icon_missing.dds"
	end

	local control = GetBuffControl(_buffName, _unitTag, _effectType)
	control.buffData.IconTexture = _iconTexture
	control.buffData.StartTime = _startTime or 0
	control.buffData.EndTime = _endTime or 0
	control.buffData.BuffName = _buffName
	control.buffData.StartDuration = startDuration
	control.buffData.active = true

	if _fadeMode then
		AUI.Fade.In(control, 500)
	else
		AUI.Fade.In(control)
	end	
		
	EVENT_MANAGER:RegisterForUpdate("AUI_Update_BuffControl" .. tostring(_unitTag) .. _buffName, 0, function() UpdateBuffTime(_buffName, _unitTag, control) end)	
	
	return control
end

local function UpdateBuffFramesLayout(_unitTag, effectType)
	SortBuffs(_unitTag, effectType)
	SetBuffsLayout(_unitTag, effectType)
	SetPreviewLayout(_unitTag, effectType)
end

local function RemoveBuff(_buffName, _unitTag, _effectType, _fadeMode)
	EVENT_MANAGER:UnregisterForUpdate("AUI_Update_BuffControl" .. tostring(_unitTag) .. _buffName)

	local control = GetBuffControl(_buffName, _unitTag, _effectType)
	if control then
		if _fadeMode then
			AUI.Fade.Out(control, 500)
		else
			AUI.Fade.Out(control)
		end
		
		control.buffData.active = false
		
		UpdateBuffFramesLayout(_unitTag, _effectType)	
	end
end

local function RemoveBuffs(_unitTag, _effectType, _fadeMode)
	local buffControls = GetBuffControls(_unitTag, _effectType)
	for _, control in pairs(buffControls) do	
		if control.buffData.active then
			local buffName = control.buffData.BuffName
			RemoveBuff(buffName, _unitTag, _effectType, _fadeMode)
		end
	end
end

local function RemoveAllBuffs()
	local buffDataList = GetBuffList()
	if buffDataList then
		for unitTag, _ in pairs(buffDataList) do
			for effectType, _ in pairs(buffDataList[unitTag]) do	
				for _, control in pairs(buffDataList[unitTag][effectType]) do
					if control.buffData.active then
						local buffName = control.buffData.BuffName	
						RemoveBuff(buffName, unitTag, effectType, false)
					end
				end
			end
		end
	end
end

local function IsBuffAllowed(_unitTag, _startTime, _endTime, _effectType, _abilityType)
	if _unitTag ~= AUI_PLAYER_UNIT_TAG and _unitTag ~= AUI_TARGET_UNIT_TAG then
		return false
	end
	
	local duration = _endTime - _startTime
	
	if _effectType == BUFF_EFFECT_TYPE_BUFF then
		if _abilityType == ABILITY_TYPE_BONUS or ABILITY_TYPE_DAMAGE then
			if _unitTag == AUI_PLAYER_UNIT_TAG then
				if not AUI.Settings.Buffs.player_show_passive_permanent_buffs and duration <= 0 then														
					return false
				elseif not AUI.Settings.Buffs.player_show_passive_time_limit_buffs and duration > 0 then
					return false
				end
			elseif _unitTag == AUI_TARGET_UNIT_TAG then
				if not AUI.Settings.Buffs.target_show_passive_permanent_buffs and duration == 0 then
					return false
				elseif not AUI.Settings.Buffs.target_show_passive_time_limit_buffs and duration > 0 then
					return false
				end
			end
		elseif _abilityType == ABILITY_TYPE_NONE then
			if _unitTag == AUI_PLAYER_UNIT_TAG and not AUI.Settings.Buffs.player_show_ability_buffs and duration > 0 then
				return false
			elseif _unitTag == AUI_TARGET_UNIT_TAG and not AUI.Settings.Buffs.target_show_ability_buffs and duration > 0 then
				return false
			end		
		end
	end
	
	return true
end

local function RefreshBuffs(_unitTag, _fadeMode)
	local numBuffs = GetNumBuffs(_unitTag) 
	local activeBuffs = {}
	
	for buffIndex = 1, numBuffs do
		local buffName, startTime, endTime, buffSlot, stackCount, iconFile, unitTag, effectType, abilityType, statusEffectType = GetUnitBuffInfo(_unitTag, buffIndex) 
		
		if buffSlot > 0 and buffName ~= "" then
			if IsBuffAllowed(_unitTag, startTime, endTime, effectType, abilityType) then
				if not IsBuffActive(_unitTag, buffName, effectType) then
					AddBuff(buffName, _unitTag, effectType, iconFile, startTime, endTime, _fadeMode)
				else
					UpdateBuff(buffName, _unitTag, effectType, iconFile, startTime, endTime)
				end
					
				if not activeBuffs[effectType] then
					activeBuffs[effectType] = {}
				end
				
				activeBuffs[effectType][buffName] = true
			end	
		end
	end	

	local buffDataList = GetUnitTagBuffList(_unitTag)
	if buffDataList then
		for effectType, _ in pairs(buffDataList) do
			for _, control in pairs(buffDataList[effectType]) do
				if control.buffData.active then
					local buffName = control.buffData.BuffName
					local effectType = control.buffData.EffectType
					if buffName and effectType then				
						if not activeBuffs[effectType] or activeBuffs[effectType] and not activeBuffs[effectType][buffName] then
							RemoveBuff(buffName, _unitTag, effectType, _fadeMode)				
						end	
					end
				end					
			end	
			
			UpdateBuffFramesLayout(_unitTag, effectType)		
		end
	end
end

function AUI.Buffs.UpdateAllBuffFramesLayout()
	if not isLoaded then
		return
	end

	local buffDataList = GetBuffList()
	if buffDataList then
		for unitTag, _ in pairs(buffDataList) do	
			for effectType, _ in pairs(buffDataList[unitTag]) do		
				UpdateBuffFramesLayout(unitTag, effectType)		
			end
		end
	end
end

function AUI.Buffs.OnEffectChanged(_unitTag)
	if not _unitTag or isPreviewShow or not isLoaded  then
		return
	end

	RefreshBuffs(_unitTag, true)
end

local function AddPreviewBuff(_buffName, _unitTag, _effectType, _textureFile, _time)
	local seconds = _time / 1000

	AddBuff(_buffName, _unitTag, _effectType, _textureFile, GetFrameTimeSeconds(), GetFrameTimeSeconds() + seconds, false)
	
	if _time > 0 then
		EVENT_MANAGER:UnregisterForUpdate("AUI_Buffs_Preview" .. _buffName)
		EVENT_MANAGER:RegisterForUpdate("AUI_Buffs_Preview" .. _buffName, _time,
		function() 
			UpdateBuff(_buffName, _unitTag, _effectType, _textureFile, GetFrameTimeSeconds(), GetFrameTimeSeconds() + seconds)
		end)
	end
	
	previewBuffList[_buffName] = true
end

local function RemovePreviewBuffs()
	for buffName, active in pairs(previewBuffList) do
		if active then
			EVENT_MANAGER:UnregisterForUpdate("AUI_Buffs_Preview" .. buffName)
			previewBuffList[buffName] = nil
		end
	end

	RemoveAllBuffs()
end

function AUI.Buffs.UpdatePreview()
	if not isLoaded  or not isPreviewShow then
		return
	end
	
	RemovePreviewBuffs()
	
	if AUI.Settings.Buffs.player_show_passive_permanent_buffs then
		AddPreviewBuff("PreviewPlayerPermPassiveBuff1", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_werewolf_010.dds", -1) -- Lykanthropie	
		AddPreviewBuff("PreviewPlayerPermPassiveBuff2", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_vampire_007.dds", -1) -- Vampire			
	end	
	
	if AUI.Settings.Buffs.player_show_ability_buffs then
		AddPreviewBuff("PreviewPlayerAbilityBuff1", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_mageguild_003.dds", 4000) -- Äquilibrium	
		AddPreviewBuff("PreviewPlayerAbilityBuff2", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_fightersguild_002.dds", 15000) -- Meisterjäger
		AddPreviewBuff("PreviewPlayerAbilityBuff3", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_undaunted_005.dds", 5000) -- Knochenschild
		AddPreviewBuff("PreviewPlayerAbilityBuff4", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_undaunted_001.dds", -1) -- Blutiger Altar
		AddPreviewBuff("PreviewPlayerAbilityBuff5", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_mageguild_002.dds", -1) -- Magierlicht	
	end	
	
	if AUI.Settings.Buffs.player_show_passive_time_limit_buffs then
		AddPreviewBuff("PreviewPlayerPermTLPassiveBuff1", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/store_tricolor_food_01.dds", 7200000) -- Bufffood (All Attributes)	
	end
	
	if AUI.Settings.Buffs.show_player_passive_permanent_debuffs then
		AddPreviewBuff("PreviewPlayerPermPassiveBuff1", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_werewolf_010.dds", -1) -- Lykanthropie	
		AddPreviewBuff("PreviewPlayerPermPassiveBuff2", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_vampire_007.dds", -1) -- Vampire		
	end
	
	if AUI.Settings.Buffs.player_show_debuffs then
		AddPreviewBuff("PreviewPlayerDebuff1", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_destructionstaff_010.dds", 10000) -- Impuls
		AddPreviewBuff("PreviewPlayerDebuff2", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_destructionstaff_001b.dds", 1300) -- Kraftschlag
		AddPreviewBuff("PreviewPlayerDebuff3", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_destructionstaff_011.dds", 21000) -- größere Verteilung
		AddPreviewBuff("PreviewPlayerDebuff4", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_otherclass_001.dds", -1) -- Seelenfalle
		AddPreviewBuff("PreviewPlayerDebuff5", AUI_PLAYER_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_undaunted_002.dds", 15000) -- Inneres Feuer
	end	
	
	if AUI.Settings.Buffs.target_show_passive_permanent_buffs then
		AddPreviewBuff("PreviewTargetPermPassiveBuff1", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_werewolf_010.dds", -1) -- Lykanthropie	
		AddPreviewBuff("PreviewTargetPermPassiveBuff2", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_vampire_007.dds", -1) -- Vampire			
	end	
	
	if AUI.Settings.Buffs.target_show_ability_buffs then
		AddPreviewBuff("PreviewTargetAbilityBuff1", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_mageguild_003.dds", 4000) -- Äquilibrium	
		AddPreviewBuff("PreviewTargetAbilityBuff2", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_fightersguild_002.dds", 15000) -- Meisterjäger
		AddPreviewBuff("PreviewTargetAbilityBuff3", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_undaunted_005.dds", 5000) -- Knochenschild
		AddPreviewBuff("PreviewTargetAbilityBuff4", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_undaunted_001.dds", -1) -- Blutiger Altar
		AddPreviewBuff("PreviewTargetAbilityBuff5", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/ability_mageguild_002.dds", -1) -- Magierlicht	
	end	
	
	if AUI.Settings.Buffs.target_show_passive_time_limit_buffs then
		AddPreviewBuff("PreviewTargetPermTLPassiveBuff1", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, "/esoui/art/icons/store_tricolor_food_01.dds", 7200000) -- Bufffood (All Attributes)	
	end	
	
	
	if AUI.Settings.Buffs.target_show_debuffs then
		AddPreviewBuff("PreviewTargetDebuff1", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_destructionstaff_010.dds", 10000) -- Impuls
		AddPreviewBuff("PreviewTargetDebuff2", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_destructionstaff_001b.dds", 1300) -- Kraftschlag
		AddPreviewBuff("PreviewTargetDebuff3", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_destructionstaff_011.dds", 21000) -- größere Verteilung
		AddPreviewBuff("PreviewTargetDebuff4", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_otherclass_001.dds", -1) -- Seelenfalle
		AddPreviewBuff("PreviewTargetDebuff5", AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, "/esoui/art/icons/ability_undaunted_002.dds", 15000) -- Inneres Feuer
	end
	
	AUI.Buffs.UpdateAllBuffFramesLayout()
end

function AUI.Buffs.ShowPreview(_show)
	if not isLoaded then
		return
	end

	AUI_Buff:SetHidden(not _show)
	
	for _, control in pairs(mainBuffControls) do
		local main = control
		local info = GetControl(main, "_Info")
		local arrow = GetControl(main, "_DirectionArrow")
		local dragOverlay = GetControl(main, "_DragOverlay")
	
		main:SetMouseEnabled(_show)	
		info:SetHidden(not _show)
		arrow:SetHidden(not _show)
		dragOverlay:SetHidden(not _show)
	end	
	
	BUFF_SCENE_FRAGMENT.hiddenReasons:SetHiddenForReason("ShouldntShow", _show)	
	
	isPreviewShow = _show
	
	if _show then
		AUI.Buffs.UpdatePreview()
	else
		RemovePreviewBuffs()
		AUI.Buffs.UpdateUI()
	end
end

function AUI.Buffs.SetToDefaultPosition(_defaultSettings)
	if not isLoaded then
		return
	end

	AUI_Buff_Player_Buffs:ClearAnchors()
	AUI_Buff_Player_Buffs:SetAnchor(_defaultSettings.player_buff_position.point, GuiRoot, _defaultSettings.player_buff_position.relativePoint, _defaultSettings.player_buff_position.offsetX, _defaultSettings.player_buff_position.offsetY)	
	_, AUI.Settings.Buffs.player_buff_position = _defaultSettings.player_buff_position

	AUI_Buff_Player_Debuffs:ClearAnchors()
	AUI_Buff_Player_Debuffs:SetAnchor(_defaultSettings.player_debuff_position.point, GuiRoot, _defaultSettings.player_debuff_position.relativePoint, _defaultSettings.player_debuff_position.offsetX, _defaultSettings.player_debuff_position.offsetY)		
	_, AUI.Settings.Buffs.player_debuff_position = _defaultSettings.player_debuff_position	
	
	AUI_Buff_Target_Buffs:ClearAnchors()
	AUI_Buff_Target_Buffs:SetAnchor(_defaultSettings.target_buff_position.point, GuiRoot, _defaultSettings.target_buff_position.relativePoint, _defaultSettings.target_buff_position.offsetX, _defaultSettings.target_buff_position.offsetY)
	_, AUI.Settings.Buffs.target_buff_position = _defaultSettings.target_buff_position	
		
	AUI_Buff_Target_Debuffs:ClearAnchors()
	AUI_Buff_Target_Debuffs:SetAnchor(_defaultSettings.target_debuff_position.point, GuiRoot, _defaultSettings.target_debuff_position.relativePoint, _defaultSettings.target_debuff_position.offsetX, _defaultSettings.target_debuff_position.offsetY)
	_, AUI.Settings.Buffs.target_debuff_position = _defaultSettings.target_debuff_position	
end

function AUI.Buffs.UpdateUI()
	if not isLoaded then
		return
	end	

	RefreshBuffs(AUI_PLAYER_UNIT_TAG, false)
	RefreshBuffs(AUI_TARGET_UNIT_TAG, false)
end

function AUI.Buffs.OnTargetChanged()
	if not isLoaded or isPreviewShow then
		return
	end
		
	if DoesUnitExist(AUI_TARGET_UNIT_TAG) then
		RefreshBuffs(AUI_TARGET_UNIT_TAG, false)
	else
		RemoveBuffs(AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_BUFF, false)
		RemoveBuffs(AUI_TARGET_UNIT_TAG, BUFF_EFFECT_TYPE_DEBUFF, false)
	end		
	
	RefreshBuffs(AUI_TARGET_UNIT_TAG, false)
	
	hasTarget = true
end

function AUI.Buffs.OnPlayerActivated()
	if not isLoaded then
		if not currentTemplate then	
			local templateName = AUI.Settings.Template.Buffs
			currentTemplate = AUI.Buffs.LoadTemplate(templateName)
		end

		CreateUI()
		
		isLoaded = true
	end
	
	AUI.Buffs.UpdateUI()
end	

function AUI.Buffs.Load()
	if isLoaded then
		return
	end

	mainBuffControls = {
		[1] = AUI_Buff_Player_Buffs,
		[2] = AUI_Buff_Player_Debuffs,
		[3] = AUI_Buff_Target_Buffs,
		[4] = AUI_Buff_Target_Debuffs,
	}
	
	BUFF_SCENE_FRAGMENT = ZO_SimpleSceneFragment:New(AUI_Buff)
	BUFF_SCENE_FRAGMENT.hiddenReasons = ZO_HiddenReasons:New()		
    BUFF_SCENE_FRAGMENT:SetConditional(function()
        return not BUFF_SCENE_FRAGMENT.hiddenReasons:IsHidden()
    end)	
	
	HUD_SCENE:AddFragment(BUFF_SCENE_FRAGMENT)
	HUD_UI_SCENE:AddFragment(BUFF_SCENE_FRAGMENT)
	SIEGE_BAR_SCENE:AddFragment(BUFF_SCENE_FRAGMENT)
	if SIEGE_BAR_UI_SCENE then
		SIEGE_BAR_UI_SCENE:AddFragment(BUFF_SCENE_FRAGMENT)
	end		
end